/*
 *	MidiPlayer1.java
 *
 *	Java Midi Tutorial.
 *
 *
 *   Copyright (c) 2003 by HarfeSoft (Harm Fesefeldt Software) <www.harfesoft.de>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published
 *   by the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
import javax.swing.*;
import javax.swing.filechooser.*;

import java.awt.*;
import java.io.File;
import java.awt.event.*;
import java.beans.*;
import	java.io.IOException;

import	javax.sound.midi.InvalidMidiDataException;
import	javax.sound.midi.MidiSystem;
import	javax.sound.midi.MidiUnavailableException;
import	javax.sound.midi.MetaEventListener;
import	javax.sound.midi.MetaMessage;
import	javax.sound.midi.Sequence;
import	javax.sound.midi.Sequencer;
import	javax.sound.midi.Synthesizer;
import	javax.sound.midi.Receiver;
import	javax.sound.midi.Transmitter;



public class MidiPlayer1 extends JPanel implements ActionListener {
    static JFrame frame;

    static String metal= "Metal";
    static String metalClassName = "javax.swing.plaf.metal.MetalLookAndFeel";

    JButton fileButton;
    JButton startButton;
    JButton stopButton;
    JButton exitButton;

    ExampleFileFilter midFilter;

    JPanel buttonPanel;

    JFileChooser chooser;

    private static Sequencer sequencer = null;

    private static Synthesizer synthesizer = null;

    Sequence sequence = null;

    public final static Dimension hpad10 = new Dimension(10,1);

    public MidiPlayer1() {
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	chooser = new JFileChooser();

	midFilter = new ExampleFileFilter("mid", "MIDI Audio Files");

        chooser.addChoosableFileFilter(midFilter);

	fileButton = new JButton("File");
	fileButton.addActionListener(this);
	startButton = new JButton("Start");
	startButton.addActionListener(this);
	stopButton = new JButton("Stop");
	stopButton.addActionListener(this);
	exitButton = new JButton("Exit");
	exitButton.addActionListener(this);



	JPanel panel = new JPanel();
	panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
	panel.add(Box.createRigidArea(hpad10));
	panel.add(fileButton);
	panel.add(Box.createRigidArea(hpad10));
	panel.add(startButton);
	panel.add(Box.createRigidArea(hpad10));
	panel.add(stopButton);
	panel.add(Box.createRigidArea(hpad10));
	panel.add(exitButton);
	panel.add(Box.createRigidArea(hpad10));

	add(panel);
	add(Box.createRigidArea(hpad10));
    }

    public void actionPerformed(ActionEvent e) {
      JComponent c = (JComponent) e.getSource();
 
      if (c == fileButton) {
          chooser.setSelectedFile(null);
	  int retval = chooser.showDialog(frame, null);
	  if (retval == JFileChooser.APPROVE_OPTION) {
	   
		File file = chooser.getSelectedFile();
		if (file != null) {
		    JOptionPane.showMessageDialog(frame,
						  "You chose this file: " +
						   file.getPath());
        
                    try {
                       sequence = MidiSystem.getSequence(file);
                    }
                    catch (InvalidMidiDataException me) {
                       JOptionPane.showMessageDialog(frame,
                                                   "This is not a MIDI File " +
                                                    file.getPath());
                    }
                    catch (IOException me) {
                       System.out.println(" Midi IO Exception");
                    }   
	        }
	   } else if (retval == JFileChooser.CANCEL_OPTION) {
	     JOptionPane.showMessageDialog(frame, "User cancelled operation. No file was chosen.");
	   } else if (retval == JFileChooser.ERROR_OPTION) {
	     JOptionPane.showMessageDialog(frame, "An error occured. No file was chosen.");
	   } else {
	     JOptionPane.showMessageDialog(frame, "Unknown operation occured.");
	   }
      } 
      else if ( c == startButton) {     
          try {
              sequencer.setSequence(sequence);
          }
          catch (InvalidMidiDataException me) {
              System.out.println(" Invalid MIDI File ");
          }    

          if(!(sequencer instanceof Synthesizer)) {
             try {
                 synthesizer = MidiSystem.getSynthesizer();
                 synthesizer.open();
                 Receiver receiver = synthesizer.getReceiver();
                 Transmitter transmitter = sequencer.getTransmitter();
                 transmitter.setReceiver(receiver);
              }    
              catch (MidiUnavailableException me) {
                 System.out.println(" Cannot connect Receiver and Transmitter");
              } 
 
          }
          sequencer.start(); 
     }
      else if (c == stopButton) {
          sequencer.stop();     
      }
      else if (c == exitButton) {
          sequencer.stop();
          sequencer.close();
          System.exit(0); 
      }
    }


    public static void main(String s[]) {


	try {
	    // UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	    UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
	} catch (Exception exc) {
	    System.out.println("Error loading L&F: " + exc);
	}

	MidiPlayer1 panel = new MidiPlayer1();

	frame = new JFrame("MidiPlayer 1");
	frame.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	});
	frame.getContentPane().add("Center", panel);
	frame.pack();
	frame.setVisible(true);
      
        try {
           sequencer = MidiSystem.getSequencer();
        } 
        catch (MidiUnavailableException me) {
           System.out.println("Midi System not available");
           System.exit(1);
        }
        if(sequencer == null) {
           System.out.println("Can't get a sequnecer");
           System.exit(1);
        }
        try {
              sequencer.open();
        }
        catch (MidiUnavailableException me) {
              System.out.println( "Can't open sequencer");
              System.exit(1);
        }

        
    }

}
