/*
 *	MidiPlayer2.java
 *
 *	Java Midi Tutorial.
 *
 *
 *   Copyright (c) 2003 by HarfeSoft (Harm Fesefeldt Software) <www.harfesoft.de>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published
 *   by the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import java.awt.*;
import java.io.File;
import java.awt.event.*;
import java.beans.*;
import	java.io.IOException;

import	javax.sound.midi.InvalidMidiDataException;
import	javax.sound.midi.MidiSystem;
import  javax.sound.midi.MidiDevice;
import	javax.sound.midi.MidiUnavailableException;
import	javax.sound.midi.MetaEventListener;
import	javax.sound.midi.MetaMessage;
import	javax.sound.midi.Sequence;
import	javax.sound.midi.Sequencer;
import	javax.sound.midi.Synthesizer;
import	javax.sound.midi.Receiver;
import	javax.sound.midi.Transmitter;
import  javax.sound.midi.Track;
import  javax.sound.midi.MidiEvent;
import  javax.sound.midi.MidiMessage;
import  javax.sound.midi.MetaMessage;


public class MidiPlayer2 extends JPanel implements ActionListener {
    static JFrame frame;

    static String metal= "Metal";
    static String metalClassName = "javax.swing.plaf.metal.MetalLookAndFeel";

    JButton fileButton;
    JButton startButton;
    JButton stopButton;
    JButton suspendButton;
    JButton resumeButton;
    JButton exitButton;
    JButton deviceButton;
    JButton infoButton;

    ExampleFileFilter midFilter;

    JPanel buttonPanel;

    JFileChooser chooser;

    private static Sequencer sequencer = null;

    private static Synthesizer synthesizer = null;

    Sequence sequence = null;
 
    int deviceIndex = -1;

    long timePosition = 0;

    String fileName = null;

    public final static Dimension hpad10 = new Dimension(10,1);


    public MidiPlayer2() {
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	chooser = new JFileChooser();

	midFilter = new ExampleFileFilter("mid", "MIDI Audio Files");

        chooser.addChoosableFileFilter(midFilter);
        
	fileButton = new JButton("File");
	fileButton.addActionListener(this);
	startButton = new JButton("Start");
	startButton.addActionListener(this);
	stopButton = new JButton("Stop");
	stopButton.addActionListener(this);
	suspendButton = new JButton("Suspend");
	suspendButton.addActionListener(this);
	resumeButton = new JButton("Resume");
	resumeButton.addActionListener(this);
	exitButton = new JButton("Exit");
	exitButton.addActionListener(this);
	deviceButton = new JButton("Device");
	deviceButton.addActionListener(this);
	infoButton = new JButton("Info");
	infoButton.addActionListener(this);



	JPanel panel = new JPanel();
	panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
	panel.add(Box.createRigidArea(hpad10));
	panel.add(fileButton);
	panel.add(Box.createRigidArea(hpad10));
	panel.add(startButton);
	panel.add(Box.createRigidArea(hpad10));
	panel.add(stopButton);
	panel.add(Box.createRigidArea(hpad10));
	panel.add(suspendButton);
	panel.add(Box.createRigidArea(hpad10));
	panel.add(resumeButton);
	panel.add(Box.createRigidArea(hpad10));
	panel.add(exitButton);
	panel.add(Box.createRigidArea(hpad10));
	panel.add(deviceButton);
	panel.add(Box.createRigidArea(hpad10));
	panel.add(infoButton);
	panel.add(Box.createRigidArea(hpad10));

	add(panel);
	add(Box.createRigidArea(hpad10));
    }

    public void actionPerformed(ActionEvent e) {
      JComponent c = (JComponent) e.getSource();
 
      if (c == fileButton) {
          chooser.setSelectedFile(null);
	  int retval = chooser.showDialog(frame, null);
	  if (retval == JFileChooser.APPROVE_OPTION) {
	   
		File file = chooser.getSelectedFile();
		if (file != null) {
		    JOptionPane.showMessageDialog(frame,
						  "You chose this file: " +
						   file.getPath());
                    fileName = file.getPath();
                    try {
                       sequence = MidiSystem.getSequence(file);
                    }
                    catch (InvalidMidiDataException me) {
                       JOptionPane.showMessageDialog(frame,
                                                   "This is not a MIDI File " +
                                                    file.getPath());
                    }
                    catch (IOException me) {
                       System.out.println(" Midi IO Exception");
                    }   
	        }
	   } else if (retval == JFileChooser.CANCEL_OPTION) {
	     JOptionPane.showMessageDialog(frame, "User cancelled operation. No file was chosen.");
	   } else if (retval == JFileChooser.ERROR_OPTION) {
	     JOptionPane.showMessageDialog(frame, "An error occured. No file was chosen.");
	   } else {
	     JOptionPane.showMessageDialog(frame, "Unknown operation occured.");
	   }
      } 
      else if ( c == startButton) {     
        if(deviceIndex < 0) {
          try {
              sequencer.setSequence(sequence);
          }
          catch (InvalidMidiDataException me) {
              System.out.println(" Invalid MIDI File ");
          }    

          if(!(sequencer instanceof Synthesizer)) {
             try {
                 synthesizer = MidiSystem.getSynthesizer();
                 synthesizer.open();
                 Receiver receiver = synthesizer.getReceiver();
                 Transmitter transmitter = sequencer.getTransmitter();
                 transmitter.setReceiver(receiver);
              }    
              catch (MidiUnavailableException me) {
                 System.out.println(" Cannot connect Receiver and Transmitter");
              } 
 
          }
          sequencer.start();
        } 
        else {          
          System.out.println("Try to get device Number " + deviceIndex);
          MidiDevice.Info[] aInfos = MidiSystem.getMidiDeviceInfo();
          MidiDevice.Info   info = aInfos[deviceIndex];
          try {
              sequencer.setSequence(sequence);
          }
          catch (InvalidMidiDataException me) {
              System.out.println(" Invalid MIDI File ");
          }    
          try {
              MidiDevice midiDevice = MidiSystem.getMidiDevice(info);
              midiDevice.open();
              Receiver receiver = midiDevice.getReceiver();
              Transmitter transmitter = sequencer.getTransmitter();
              transmitter.setReceiver(receiver);
          }
          catch (MidiUnavailableException me)
          {
              System.out.println("Can't build up Midi System");
          }
          sequencer.start();
        }
      }
      else if (c == stopButton) {
          sequencer.stop();     
      }
      else if(c == suspendButton) {
          timePosition = sequencer.getMicrosecondPosition();
          sequencer.stop();
      }
      else if(c == resumeButton) {
          sequencer.setMicrosecondPosition(timePosition);
          sequencer.start();
      }
      else if (c == exitButton) {
          sequencer.stop();
          sequencer.close();
          System.exit(0); 
      }
      else if (c == deviceButton) {
      	  MidiDevice.Info[]	aInfos = MidiSystem.getMidiDeviceInfo();
          Object[][] data = new Object[aInfos.length][5];
          
	  for (int i = 0; i < aInfos.length; i++) {
	 	try {
			MidiDevice	device = MidiSystem.getMidiDevice(aInfos[i]);
			boolean		bAllowsInput = (device.getMaxTransmitters() != 0);
			boolean		bAllowsOutput = (device.getMaxReceivers() != 0);
			if (bAllowsInput || bAllowsOutput) {
				System.out.println(""+i+"  "
					+(bAllowsInput?"IN ":"   ")
					+(bAllowsOutput?"OUT ":"    ")
					+aInfos[i].getName()+", "
					+aInfos[i].getVendor()+", "
					+aInfos[i].getVersion()+", "
					+aInfos[i].getDescription());
                                data[i][0] = (bAllowsInput?"IN ":"   ") 
                                            +(bAllowsOutput?"OUT":"   ");
                                data[i][1] = aInfos[i].getName();
                                data[i][2] = aInfos[i].getVendor();
                                data[i][3] = aInfos[i].getVersion();
                                data[i][4] = aInfos[i].getDescription();
			}
		}
		catch (MidiUnavailableException me) {
			// device is obviously not available...
		}
	  }
	  if (aInfos.length == 0) {
		System.out.println("[No devices available]");
                System.exit(0);
	  }
          String[] columnNames = {"Mode","Name","Vendor","Version","Description"};
 
          JTable table = new JTable(data,columnNames);
          JScrollPane scrollPane = new JScrollPane(table);
          JFrame deviceFrame = new JFrame("Midi Devices");
          deviceFrame.getContentPane().add(scrollPane);
          deviceFrame.pack();
          Dimension frameDimension = new Dimension(500,100);
          deviceFrame.setSize(frameDimension);
          deviceFrame.setVisible(true);
          table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
          ListSelectionModel rowSM = table.getSelectionModel();
          rowSM.addListSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent e) {
                    //Ignore extra messages.
                    if (e.getValueIsAdjusting()) return;
                    
                    ListSelectionModel lsm = (ListSelectionModel)e.getSource();
                    if (lsm.isSelectionEmpty()) {
                        System.out.println("No rows are selected.");
                    } else {
                        int selectedRow = lsm.getMinSelectionIndex();
                        System.out.println("Row " + selectedRow
                                           + " is now selected.");
                        deviceIndex = selectedRow; 
                    }
                }
          });
 
      }
      else if(c == infoButton) {
 
         if(sequence == null) {
            System.out.println("No Sequence defined");
         }
         else {
            Object[][] fileInfo = new Object[6][2];
            int lastIndex = fileName.lastIndexOf("/");
            if (lastIndex < 0) lastIndex = fileName.length()-12;
            if (lastIndex < 0) lastIndex = 0; 
            if(fileName != null) fileInfo[0][1] = "......" +
                 fileName.substring(lastIndex,fileName.length());
            else fileInfo[0][1] = "unknown";
            fileInfo[1][1] = sequencer.getMicrosecondLength() + " MicroSeconds";
            float divType = sequence.getDivisionType();
            if(      divType == Sequence.PPQ)      fileInfo[2][1] = "PPQ ";
            else if (divType == Sequence.SMPTE_24) fileInfo[2][1] = "SMPTE, 24 frames/sec.";
            else if (divType == Sequence.SMPTE_25) fileInfo[2][1] = "SMPTE, 25 frames/sec.";
            else if (divType == Sequence.SMPTE_30DROP) fileInfo[2][1] = "SMPTE, 29.97 frames/sec.";
            else if (divType == Sequence.SMPTE_30) fileInfo[2][1] = "SMPTE, 30 frames/sec.";
            else                                   fileInfo[2][1] = "unknown";
            String resType = null;
            if(sequence.getDivisionType() == Sequence.PPQ) resType = " ticks/beat";
            else                                           resType = " ticks/frame";
            fileInfo[3][1] = sequence.getResolution() + resType;   
            Track[] tracks = sequence.getTracks();
            int nInfos = 2;
            for(int nTrack=0; nTrack<tracks.length; nTrack++) {
               Track track = tracks[nTrack];
               for(int nEvent=0; nEvent<track.size(); nEvent++) {
                  MidiEvent event = track.get(nEvent);
                  MidiMessage message = event.getMessage();
                  if(message instanceof MetaMessage) {
                     MetaMessage meta = (MetaMessage)message;
                     byte[] abMessage = meta.getMessage();
                     byte[] abData = meta.getData();
                     int    nDataLength = meta.getLength();
                     String strMessage = null;
                     if(meta.getType() == 1) {
                        fileInfo[3][1] = new String(abData);
                        nInfos++;
                     }   
                     else if(meta.getType() == 2) {
                        fileInfo[4][1] = new String(abData);
                        nInfos++;
                     }     
                    else if(meta.getType() == 3) {
                        fileInfo[5][1] = new String(abData);
                        nInfos++;
                     }                                                  
                  }
                  if(nInfos == 5) break;
               }
               if(nInfos == 5) break;
            }
            String[] columnNames = {"Info","Value"};
            fileInfo[0][0] = "File Name";
            fileInfo[1][0] = "Duration";
            fileInfo[2][0] = "Division";
            fileInfo[3][0] = "Text";
            fileInfo[4][0] = "Copyright";
            fileInfo[5][0] = "Sequence Name";
            JTable table = new JTable(fileInfo,columnNames);
            JScrollPane scrollPane = new JScrollPane(table);
            JFrame infoFrame = new JFrame("File Info");
            infoFrame.getContentPane().add(scrollPane);
            infoFrame.pack();
            Dimension frameDimension = new Dimension(500,150);
            infoFrame.setSize(frameDimension);
            infoFrame.setVisible(true);              
         }
      }
    }




      public static void main(String s[]) {


	try {
	    // UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	    UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
	} catch (Exception exc) {
	    System.out.println("Error loading L&F: " + exc);
	}

	MidiPlayer2 panel = new MidiPlayer2();

	frame = new JFrame("MidiPlayer 2");
	frame.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	});
	frame.getContentPane().add("Center", panel);
	frame.pack();
	frame.setVisible(true);
      
        try {
           sequencer = MidiSystem.getSequencer();
        } 
        catch (MidiUnavailableException me) {
           System.out.println("Midi System not available");
           System.exit(1);
        }
        if(sequencer == null) {
           System.out.println("Can't get a sequnecer");
           System.exit(1);
        }
        try {
              sequencer.open();
        }
        catch (MidiUnavailableException me) {
              System.out.println( "Can't open sequencer");
              System.exit(1);
        }       
     }
}


