/*
 *	MidiPlayer3.java
 *
 *	Java Midi Tutorial 
 *
 *
 *   Copyright (c) 2003 by HarfeSoft (Harm Fesefeldt Software) <www.harfesoft.de>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published
 *   by the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.event.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.ImageIcon;

import java.awt.*;
import java.io.File;
import java.awt.event.*;
import java.beans.*;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

import  javax.sound.midi.*;
import	javax.sound.midi.InvalidMidiDataException;
import	javax.sound.midi.MidiSystem;
import  javax.sound.midi.MidiDevice;
import	javax.sound.midi.MidiUnavailableException;
import	javax.sound.midi.MetaEventListener;
import	javax.sound.midi.MetaMessage;
import	javax.sound.midi.Sequence;
import	javax.sound.midi.Sequencer;
import	javax.sound.midi.Synthesizer;
import	javax.sound.midi.Receiver;
import	javax.sound.midi.Transmitter;
import  javax.sound.midi.Track;
import  javax.sound.midi.MidiEvent;
import  javax.sound.midi.MidiMessage;
import  javax.sound.midi.MetaMessage;
import  javax.sound.midi.MidiChannel;



public class MidiPlayer3 extends JPanel implements ActionListener {
    static JFrame frame;

    static String metal= "Metal";
    static String metalClassName = "javax.swing.plaf.metal.MetalLookAndFeel";

    JButton fileButton;
    JButton startButton;
    JButton stopButton;
    JButton suspendButton;
    JButton resumeButton;
    JButton exitButton;
    JButton deviceButton;
    JButton infoButton;
    JButton controlButton;
    JButton gmButton;

    JPanel[] chanPan = new JPanel[17];
    JSlider[] gainSlider = new JSlider[16];
    JSlider[] panSlider= new JSlider[16];
    JSlider progSlider;
    JSlider masterVol; 
    JPanel volControl;
    JPanel progPanel;
    public JButton textButton;
    JLabel progLabel;
    JLabel masterLabel;
    JLabel[] gainLabel = new JLabel[16];
    JLabel[] panLabel = new JLabel[16];

    ChangeListener listener;
    
    ExampleFileFilter midFilter;
    ExampleFileFilter gmFilter;

    JPanel buttonPanel;

    JFileChooser chooser;
    JFileChooser gmChooser;

    MidiDevice midiDevice;

    private static Sequencer sequencer = null;

    private static Synthesizer synthesizer = null;

    private static Soundbank soundbank = null;

    private static List openedMidiDeviceList;

    private static MidiChannel channels[];

    Sequence sequence = null;
 
    int deviceIndex = -1;

    long duration = 0;

    String fileName = null;

    public final static Dimension hpad10 = new Dimension(10,1);
    public final static Dimension hpad20 = new Dimension(20,1);


    public MidiPlayer3() {
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        Dimension frameDim = new Dimension(670,40);
        setSize(frameDim);        

	chooser = new JFileChooser();
        gmChooser = new JFileChooser();

	midFilter = new ExampleFileFilter("mid", "MIDI Audio Files");
        gmFilter = new ExampleFileFilter("gm", "Headspace Soundbank");

        chooser.addChoosableFileFilter(midFilter);
        gmChooser.addChoosableFileFilter(gmFilter);

        openedMidiDeviceList = new ArrayList();
        
        ImageIcon fileIcon = createImageIcon("Images/file.gif");
        ImageIcon startIcon = createImageIcon("Images/start.gif");
        ImageIcon stopIcon = createImageIcon("Images/stop.gif");
        ImageIcon suspendIcon = createImageIcon("Images/suspend.gif");
        ImageIcon resumeIcon = createImageIcon("Images/resume.gif");
        ImageIcon exitIcon = createImageIcon("Images/exit.gif");
        ImageIcon deviceIcon = createImageIcon("Images/device.gif");
        ImageIcon infoIcon = createImageIcon("Images/info.gif");
        ImageIcon controlsIcon = createImageIcon("Images/controls.gif");
        ImageIcon soundbankIcon = createImageIcon("Images/soundbank.gif");

	fileButton = new JButton(fileIcon);
	fileButton.addActionListener(this);
        fileButton.setToolTipText("Select File");
	startButton = new JButton(startIcon);
	startButton.addActionListener(this);
        startButton.setToolTipText("Start Playback");
	stopButton = new JButton(stopIcon);
	stopButton.addActionListener(this);
        stopButton.setToolTipText("Stop Playback");
	suspendButton = new JButton(suspendIcon);
	suspendButton.addActionListener(this);
        suspendButton.setToolTipText("Pause Playback");
	resumeButton = new JButton(resumeIcon);
	resumeButton.addActionListener(this);
        resumeButton.setToolTipText("Continue Playback");        
	exitButton = new JButton(exitIcon);
	exitButton.addActionListener(this);
        exitButton.setToolTipText("Exit MIDI Player");
	deviceButton = new JButton(deviceIcon);
	deviceButton.addActionListener(this);
        deviceButton.setToolTipText("Select MIDI Device");        
	infoButton = new JButton(infoIcon);
	infoButton.addActionListener(this);
        infoButton.setToolTipText("Display File Info");
	controlButton = new JButton(controlsIcon);
	controlButton.addActionListener(this);
        controlButton.setToolTipText("Volume Control");
        textButton = new JButton("   0");
        textButton.addActionListener(this);
        textButton.setToolTipText("Display Timing");       
        gmButton = new JButton(soundbankIcon);
        gmButton.addActionListener(this);
        gmButton.setToolTipText("Select Soundfonts");

         masterVol = new JSlider(JSlider.VERTICAL);
         masterLabel = new JLabel("Volume");
         masterVol.setToolTipText("Master Volume");
         listener = new SliderListener(masterLabel);
         masterVol.addChangeListener(listener);
         masterVol.setMajorTickSpacing(20);
         masterVol.setMinorTickSpacing(10);
         masterVol.setPaintTicks(true);
         masterVol.setPaintLabels(true);
         masterVol.setValueIsAdjusting(true);
         progSlider = new JSlider(JSlider.HORIZONTAL, 0,100,0);
         progLabel = new JLabel("Progress");
         progSlider.setToolTipText("Playback Progress");
         listener = new SliderListener(progLabel); 
         progSlider.addChangeListener(listener);
         progSlider.setMajorTickSpacing(20);
         progSlider.setMinorTickSpacing(10);
         progSlider.setPaintTicks(true);
         progSlider.setPaintLabels(true);
         progSlider.setValueIsAdjusting(true);

         for(int i = 0; i<16; i++) {
            gainSlider[i] = new JSlider(JSlider.VERTICAL,0,100,50);
            gainLabel[i] = new JLabel(""+i);
            gainSlider[i].setToolTipText("Volume Channel " + i);
            listener = new SliderListener(gainLabel[i]);
            gainSlider[i].addChangeListener(listener);
            gainSlider[i].setValueIsAdjusting(true);
            panSlider[i] = new JSlider(JSlider.VERTICAL,0,100,50);
            int j = 16 + i;
            panLabel[i] = new JLabel(""+j);
            panSlider[i].setToolTipText("Pan Channel " +i); 
            listener = new SliderListener(panLabel[i]);
            panSlider[i].addChangeListener(listener);
            panSlider[i].setValueIsAdjusting(true);

         }
 
	JPanel panel = new JPanel();
	panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
        panel.setSize(frameDim);
	panel.add(fileButton);
	panel.add(startButton);
	panel.add(stopButton);
	panel.add(suspendButton);
	panel.add(resumeButton);
	panel.add(exitButton);
	panel.add(deviceButton);
	panel.add(infoButton);
	panel.add(controlButton);
	panel.add(gmButton);
	add(panel);

    }
 


    public void actionPerformed(ActionEvent e) {
      JComponent c = (JComponent) e.getSource();
 
      if (c == fileButton) {
          chooser.setSelectedFile(null);
	  int retval = chooser.showDialog(frame, null);
	  if (retval == JFileChooser.APPROVE_OPTION) {
	   
		File file = chooser.getSelectedFile();
		if (file != null) {
		    JOptionPane.showMessageDialog(frame,
						  "You chose this file: " +
						   file.getPath());
                    fileName = file.getPath();
                    try {
                       sequence = MidiSystem.getSequence(file);
                    }
                    catch (InvalidMidiDataException me) {
                       JOptionPane.showMessageDialog(frame,
                                                   "This is not a MIDI File " +
                                                    file.getPath());
                    }
                    catch (IOException me) {
                       System.out.println(" Midi IO Exception");
                    }   
	        }
	   } else if (retval == JFileChooser.CANCEL_OPTION) {
	     JOptionPane.showMessageDialog(frame, "User cancelled operation. No file was chosen.");
	   } else if (retval == JFileChooser.ERROR_OPTION) {
	     JOptionPane.showMessageDialog(frame, "An error occured. No file was chosen.");
	   } else {
	     JOptionPane.showMessageDialog(frame, "Unknown operation occured.");
	   }
      } 
      else if ( c == startButton) {     
        if(deviceIndex < 0) {
          try {
              sequencer.setSequence(sequence);
          }
          catch (InvalidMidiDataException me) {
              System.out.println(" Invalid MIDI File ");
          }    

          if(!(sequencer instanceof Synthesizer)) {
             try {
                 synthesizer = MidiSystem.getSynthesizer();
                 channels = synthesizer.getChannels();
                 synthesizer.open();
                 Receiver receiver = synthesizer.getReceiver();
                 Transmitter transmitter = sequencer.getTransmitter();
                 transmitter.setReceiver(receiver);
              }    
              catch (MidiUnavailableException me) {
                 System.out.println(" Cannot connect Receiver and Transmitter");
              } 
 
          } else {
  
              synthesizer = (Synthesizer) sequencer;
              channels = synthesizer.getChannels();
          }  
          textButton.setText("   0");
          progSlider.setValue(0);
          duration = sequencer.getMicrosecondLength(); 
          for(int i = 0; i<16; i++) {
             gainSlider[i].setValue((int)(channels[i].getController(7)*100./127));
             panSlider[i].setValue((int)(channels[i].getController(10)*100./127));
          }
          sequencer.start();
        } 
        else { 
          System.out.println("Try to get device Number " + deviceIndex);
          MidiDevice.Info[] aInfos = MidiSystem.getMidiDeviceInfo();
          MidiDevice.Info   info = aInfos[deviceIndex];
          try {
              sequencer.setSequence(sequence);
          }
          catch (InvalidMidiDataException me) {
              System.out.println(" Invalid MIDI File ");
          }    
          try {
              MidiDevice midiDevice = MidiSystem.getMidiDevice(info);
              openedMidiDeviceList.add(midiDevice);
              midiDevice.open();
              Receiver receiver = midiDevice.getReceiver();
              Transmitter transmitter = sequencer.getTransmitter();
              transmitter.setReceiver(receiver);
              
          }
          catch (MidiUnavailableException me)
          {
              System.out.println("Can't build up Midi System");
          }

          if(!(sequencer instanceof Synthesizer)) {
             try {
                 synthesizer = MidiSystem.getSynthesizer();
                 channels = synthesizer.getChannels();
              }    
              catch (MidiUnavailableException me) {
                 System.out.println(" Cannot get synthesizer");
              } 
 
          } else {
  
              synthesizer = (Synthesizer) sequencer;
              channels = synthesizer.getChannels();
          } 
          textButton.setText("   0");
          progSlider.setValue(0);
          duration = sequencer.getMicrosecondLength();
          for(int i = 0; i<16; i++) {
             gainSlider[i].setValue((int)(channels[i].getController(7)*100./127));
             panSlider[i].setValue((int)(channels[i].getController(10)*100./127));
          }
          sequencer.start();
        }
      }
      else if (c == stopButton) {
          sequencer.stop(); 
      }
      else if(c == suspendButton) {
          sequencer.stop();

      }
      else if(c == resumeButton) {
          sequencer.start();
      }
      else if (c == exitButton) {
          sequencer.stop();
          sequencer.close();
          Iterator iterator = openedMidiDeviceList.iterator();
          while(iterator.hasNext()) {
             MidiDevice device = (MidiDevice) iterator.next();
             device.close();
          }
          System.exit(0); 
      }
      else if (c == deviceButton) {
      	  MidiDevice.Info[]	aInfos = MidiSystem.getMidiDeviceInfo();
          Object[][] data = new Object[aInfos.length][5];
          
	  for (int i = 0; i < aInfos.length; i++) {
	 	try {
			MidiDevice	device = MidiSystem.getMidiDevice(aInfos[i]);
			boolean		bAllowsInput = (device.getMaxTransmitters() != 0);
			boolean		bAllowsOutput = (device.getMaxReceivers() != 0);
			if (bAllowsInput || bAllowsOutput) {
				System.out.println(""+i+"  "
					+(bAllowsInput?"IN ":"   ")
					+(bAllowsOutput?"OUT ":"    ")
					+aInfos[i].getName()+", "
					+aInfos[i].getVendor()+", "
					+aInfos[i].getVersion()+", "
					+aInfos[i].getDescription());
                                data[i][0] = (bAllowsInput?"IN ":"   ") 
                                            +(bAllowsOutput?"OUT":"   ");
                                data[i][1] = aInfos[i].getName();
                                data[i][2] = aInfos[i].getVendor();
                                data[i][3] = aInfos[i].getVersion();
                                data[i][4] = aInfos[i].getDescription();
			}
		}
		catch (MidiUnavailableException me) {
			// device is obviously not available...
		}
	  }
	  if (aInfos.length == 0) {
		System.out.println("[No devices available]");
                System.exit(0);
	  }
          String[] columnNames = {"Mode","Name","Vendor","Version","Description"};
 
          JTable table = new JTable(data,columnNames);
          JScrollPane scrollPane = new JScrollPane(table);
          JFrame deviceFrame = new JFrame("Midi Devices");
          deviceFrame.getContentPane().add(scrollPane);
          deviceFrame.pack();
          Dimension frameDimension = new Dimension(670,100);
          deviceFrame.setSize(frameDimension);
          deviceFrame.setVisible(true);
          table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
          ListSelectionModel rowSM = table.getSelectionModel();
          rowSM.addListSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent e) {
                    //Ignore extra messages.
                    if (e.getValueIsAdjusting()) return;
                    
                    ListSelectionModel lsm = (ListSelectionModel)e.getSource();
                    if (lsm.isSelectionEmpty()) {
                        System.out.println("No rows are selected.");
                    } else {
                        int selectedRow = lsm.getMinSelectionIndex();
                        System.out.println("Row " + selectedRow
                                           + " is now selected.");
                        deviceIndex = selectedRow; 
                    }
                }
          });
 
      }
      else if(c == infoButton) {
 
         if(sequence == null) {
            System.out.println("No Sequence defined");
         }
         else {
            Object[][] fileInfo = new Object[6][2];
            int lastIndex = fileName.lastIndexOf("/");
            if (lastIndex < 0) lastIndex = fileName.length()-12;
            if (lastIndex < 0) lastIndex = 0; 
            if(fileName != null) fileInfo[0][1] = "......" +
                 fileName.substring(lastIndex,fileName.length());
            else fileInfo[0][1] = "unknown";
            fileInfo[1][1] = sequencer.getMicrosecondLength() + " MicroSeconds";
            float divType = sequence.getDivisionType();
            if(      divType == Sequence.PPQ)      fileInfo[2][1] = "PPQ ";
            else if (divType == Sequence.SMPTE_24) fileInfo[2][1] = "SMPTE, 24 frames/sec.";
            else if (divType == Sequence.SMPTE_25) fileInfo[2][1] = "SMPTE, 25 frames/sec.";
            else if (divType == Sequence.SMPTE_30DROP) fileInfo[2][1] = "SMPTE, 29.97 frames/sec.";
            else if (divType == Sequence.SMPTE_30) fileInfo[2][1] = "SMPTE, 30 frames/sec.";
            else                                   fileInfo[2][1] = "unknown";
            String resType = null;
            if(sequence.getDivisionType() == Sequence.PPQ) resType = " ticks/beat";
            else                                           resType = " ticks/frame";
            fileInfo[3][1] = sequence.getResolution() + resType;   
            Track[] tracks = sequence.getTracks();
            int nInfos = 2;
            for(int nTrack=0; nTrack<tracks.length; nTrack++) {
               Track track = tracks[nTrack];
               for(int nEvent=0; nEvent<track.size(); nEvent++) {
                  MidiEvent event = track.get(nEvent);
                  MidiMessage message = event.getMessage();
                  if(message instanceof MetaMessage) {
                     MetaMessage meta = (MetaMessage)message;
                     byte[] abMessage = meta.getMessage();
                     byte[] abData = meta.getData();
                     int    nDataLength = meta.getLength();
                     String strMessage = null;
                     if(meta.getType() == 1) {
                        fileInfo[3][1] = new String(abData);
                        nInfos++;
                     }   
                     else if(meta.getType() == 2) {
                        fileInfo[4][1] = new String(abData);
                        nInfos++;
                     }     
                    else if(meta.getType() == 3) {
                        fileInfo[5][1] = new String(abData);
                        nInfos++;
                     }                                                  
                  }
                  if(nInfos == 5) break;
               }
               if(nInfos == 5) break;
            }
            String[] columnNames = {"Info","Value"};
            fileInfo[0][0] = "File Name";
            fileInfo[1][0] = "Duration";
            fileInfo[2][0] = "Division";
            fileInfo[3][0] = "Text";
            fileInfo[4][0] = "Copyright";
            fileInfo[5][0] = "Sequence Name";
            JTable table = new JTable(fileInfo,columnNames);
            JScrollPane scrollPane = new JScrollPane(table);
            JFrame infoFrame = new JFrame("File Info");
            infoFrame.getContentPane().add(scrollPane);
            infoFrame.pack();
            Dimension frameDimension = new Dimension(670,150);
            infoFrame.setSize(frameDimension);
            infoFrame.setVisible(true);              
         }
      }
      else if(c == controlButton) {
 
         JFrame controlFrame = new JFrame("Controls");
         Dimension frameDimension = new Dimension(670,400);
         controlFrame.setSize(frameDimension);
         JPanel progPanel = new JPanel();
         Dimension progDim = new Dimension(670,150);
         progPanel.setSize(progDim);
         progPanel.add(progSlider);
         progPanel.add(textButton);
         Dimension chanDim = new Dimension(40,150);
         Dimension volDim = new Dimension(670,150);
         JPanel volControl = new JPanel();
         volControl.setLayout(new BoxLayout(volControl,BoxLayout.X_AXIS));
         volControl.setSize(volDim); 
	 volControl.add(Box.createRigidArea(hpad20));
         JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                                               progPanel,volControl);
         splitPane.setContinuousLayout(true);
         for(int i=0; i<16; i++) {
            chanPan[i] = new JPanel();
            chanPan[i].setSize(chanDim);
            chanPan[i].setLayout(new BoxLayout(chanPan[i],BoxLayout.Y_AXIS));
            chanPan[i].add(panSlider[i]);
            chanPan[i].add(gainSlider[i]);
            volControl.add(chanPan[i]);
            volControl.add(Box.createRigidArea(hpad20));
         }
         chanPan[16] = new JPanel();
         chanPan[16].setSize(chanDim);
         chanPan[16].setLayout(new BoxLayout(chanPan[16],BoxLayout.Y_AXIS));
         chanPan[16].add(Box.createRigidArea(hpad20));
         chanPan[16].add(Box.createRigidArea(hpad20));
         chanPan[16].add(masterVol);
         volControl.add(chanPan[16]);
         controlFrame.getContentPane().add(splitPane);
         controlFrame.setVisible(true);
                 
      }
      else if(c == textButton) {
         if(duration > 0) {
             long i = sequencer.getMicrosecondPosition();
             int j = (int)(i*100./duration);
             progSlider.setValue(j);
             j = (int)(i/1000000.);
             textButton.setText("" + j);
         }
      }
      else if (c == gmButton) {
          chooser.setSelectedFile(null);
	  int retval = gmChooser.showDialog(frame, null);
	  if (retval == JFileChooser.APPROVE_OPTION) {
	   
		File file = gmChooser.getSelectedFile();
		if (file != null) {
		    JOptionPane.showMessageDialog(frame,
						  "You chose this file: " +
						   file.getPath());
                    fileName = file.getPath();
                    try {
                       soundbank = MidiSystem.getSoundbank(file);
                    }
                    catch (InvalidMidiDataException me) {
                       JOptionPane.showMessageDialog(frame,
                                                   "This is not a GM File " +
                                                    file.getPath());
                    }
                    catch (IOException me) {
                       System.out.println(" Midi IO Exception");
                    }   
	        }
	   } else if (retval == JFileChooser.CANCEL_OPTION) {
	     JOptionPane.showMessageDialog(frame, "User cancelled operation. No file was chosen.");
	   } else if (retval == JFileChooser.ERROR_OPTION) {
	     JOptionPane.showMessageDialog(frame, "An error occured. No file was chosen.");
	   } else {
	     JOptionPane.showMessageDialog(frame, "Unknown operation occured.");
	   }
      } 

    }
 
      protected static ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = MidiPlayer3.class.getResource(path);
        if(imgURL != null) {
           return new ImageIcon(imgURL);
        } else {
           return null;
        }
      }

      public static void main(String s[]) {


	try {
	    // UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	    UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
	} catch (Exception exc) {
	    System.out.println("Error loading L&F: " + exc);
	}

	MidiPlayer3 panel = new MidiPlayer3();

	frame = new JFrame("HarfeSoft MidiPlayer");
	frame.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	});
	frame.getContentPane().add("Center", panel);
	frame.pack();
	frame.setVisible(true);
     
        try {
           sequencer = MidiSystem.getSequencer();
        } 
        catch (MidiUnavailableException me) {
           System.out.println("Midi System not available");
           System.exit(1);
        }
        if(sequencer == null) {
           System.out.println("Can't get a sequnecer");
           System.exit(1);
        }
        sequencer.addMetaEventListener(new MetaEventListener()
            {
               public void meta(MetaMessage event)
               {
                  if(event.getType() == 47) 
                  {
                     sequencer.stop();
                     System.out.println("EOF sequencer.stop");
                  }
               }
            }
        );
        try {
              sequencer.open();
        }
        catch (MidiUnavailableException me) {
              System.out.println( "Can't open sequencer");
              System.exit(1);
        }       
     }

     class SliderListener implements ChangeListener {
         JLabel label;
         long timePos;
         public SliderListener (JLabel f) {
            label = f;
         }
         public void stateChanged(ChangeEvent e) {
            JSlider slider = (JSlider)e.getSource(); 
            if(slider.getValueIsAdjusting()) return;  
            if(channels != null) { 
              if(label.getText().equals("Volume"))  {
                 System.out.println("Master Volume Control");
                 for(int i = 0; i<channels.length; i++) {
                    int chanvol = 127;
                    if(i<16) chanvol = gainSlider[i].getValue();
                    channels[i].controlChange(7,(int)(slider.getValue()*chanvol/100.));    
                 } 
       
              }
              else if(label.getText().equals("Progress")) {
                 if(sequencer != null) {
                    timePos = (long)(slider.getValue()*sequencer.getMicrosecondLength()/100.);
                    System.out.println("Progress Control");
                    sequencer.setMicrosecondPosition(timePos);
                    textButton.setText(""+(int)(timePos/1000000.));
                 }   
              }
              else {
                 int ch = Integer.valueOf(label.getText()).intValue();
                 System.out.println("Channel Control " + ch);
                 if(ch >=0 && ch < 16) {
                    channels[ch].controlChange(7,(int)(slider.getValue()*127/100.));
                 }
                 else {
                    channels[ch-16].controlChange(10,(int)(slider.getValue()*127/100.));  
                 }
              }
            }
            else {
//               System.out.println(" No Midi Channels available");
            }
         }
     }
}


